
package com.dkha.common.util;


import cn.afterturn.easypoi.excel.ExcelExportUtil;
import cn.afterturn.easypoi.excel.entity.ExportParams;
import com.dkha.common.exception.DkException;
import com.dkha.common.exception.EmployeeException;
import io.swagger.annotations.ApiModelProperty;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.hssf.usermodel.HSSFDateUtil;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.core.io.ClassPathResource;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;
import java.io.InputStream;
import java.io.OutputStream;
import java.lang.reflect.Field;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.List;

import static org.apache.poi.ss.usermodel.Cell.CELL_TYPE_BLANK;

/**
 * excel工具类
 *
 * @author Mark sunlightcs@gmail.com
 */
public class ExcelUtils {
    private static Logger logger = LoggerFactory.getLogger(ExcelUtils.class);

    /**
     * 获取workbook对象
     *
     * @param multipartFile
     * @return
     * @throws Exception
     */
    public static Workbook getWorkbook(MultipartFile multipartFile) throws Exception {
        if (multipartFile.isEmpty()) {
            throw new EmployeeException("文件不存在");
        }
        if (!multipartFile.getOriginalFilename().endsWith("xls") && !multipartFile.getOriginalFilename().endsWith("xlsx")) {

            throw new EmployeeException("请上传excel文件");
        }
        Workbook workbook = null;
        InputStream is = multipartFile.getInputStream();
        if (multipartFile.getOriginalFilename().endsWith("xls")) {
            workbook = new HSSFWorkbook(is);
        }
        if (multipartFile.getOriginalFilename().endsWith("xlsx")) {
            workbook = new XSSFWorkbook(is);
        }
        return workbook;
    }


    /**
     * Excel导出
     *
     * @param response  response
     * @param fileName  文件名
     * @param list      数据List
     * @param pojoClass 对象Class
     */
    public static <T> void exportExcel(HttpServletResponse response, String fileName, ArrayList<T> list,
                                       Class<T> pojoClass) throws Exception {
        if (StringUtils.isBlank(fileName)) {
            //当前日期
            fileName = DateUtils.formatDate(new Date());
        }
        Workbook workbook = ExcelExportUtil.exportExcel(new ExportParams(), pojoClass, list);
        write(response, workbook, fileName);

    }

    /**
     * 导出excel表头
     *
     * @param response
     * @param pojo
     * @param <T>
     */
    public static <T> Workbook downloadTemplate(HttpServletResponse response, Class<T> pojo) throws Exception {
        String[] swaggerValues = getSwaggerValue(pojo);
        Workbook workbook = new HSSFWorkbook();
        Sheet sheet = workbook.createSheet();
        Row row = sheet.createRow(0);
        for (int i = 0; i < swaggerValues.length; i++) {
            row.createCell(i).setCellValue(swaggerValues[i]);
        }
//        //锁定第一行无法修改
//        sheet.createFreezePane(0, 1, 0, 1);

        return workbook;
    }


    /**
     * 获取swagger属性值
     *
     * @param pojoClass
     * @return
     * @author hechenggang
     */
    public static String[] getSwaggerValue(Class<?> pojoClass) {
        Field[] fields = pojoClass.getDeclaredFields();
        Field field;
        int length = fields.length;
        String[] value = new String[length];
        for (int i = 0; i < length; i++) {
            fields[i].setAccessible(true);
        }
        for (int i = 0; i < length; i++) {
            try {
                field = pojoClass.getDeclaredField(fields[i].getName());
                ApiModelProperty apiModelProperty = field.getAnnotation(ApiModelProperty.class);
                if (apiModelProperty != null) {
                    value[i] = apiModelProperty.value();
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        return value;
    }


    /**
     * //     * Excel导出，先sourceList转换成List<targetClass>，再导出
     * //     *
     * //     * @param response    response
     * //     * @param fileName    文件名
     * //     * @param sourceList  原数据List
     * //     * @param targetClass 目标对象Class
     * //
     */
//    public static void exportExcelToTarget(HttpServletResponse response, String fileName, Collection<?> sourceList,
//                                           Class<?> targetClass) throws Exception {
//        List targetList = new ArrayList<>(sourceList.size());
//        for (Object source : sourceList) {
//            Object target = targetClass.newInstance();
//            BeanUtils.copyProperties(source, target);
//            targetList.add(target);
//        }
//
//        exportExcel(response, fileName, targetList, targetClass);
//    }

//    public static List<Map> readExcel(MultipartFile file, int sheetNo, int startRowNo, int startColNo, int fieldRowNo)
//            throws Exception {
//        Workbook wb = null;
//        String fileName = file.getOriginalFilename();
//        String extString = StringUtils.substringAfterLast(fileName, ".");
//        if ("xls".equalsIgnoreCase(extString)) {
//            wb = new HSSFWorkbook(file.getInputStream());
//        } else if ("xlsx".equalsIgnoreCase(extString)) {
//            wb = new XSSFWorkbook(file.getInputStream());
//        } else {
//            throw new Exception("请上传Excel文件！");
//        }
//        Map map = null;
//        Row row = null;
//        Sheet sheet = wb.getSheetAt(sheetNo);
//        Row fieldRow = sheet.getRow(fieldRowNo);
//        int rowNum = sheet.getPhysicalNumberOfRows();
//        int colNum = fieldRow.getPhysicalNumberOfCells();
//        List<Map> datas = new ArrayList<Map>();
//        for (int i = startRowNo; i <= rowNum; i++) {
//            map = new HashMap();
//            row = sheet.getRow(i);
//            for (int j = startColNo; j < colNum; j++) {
//                map.put(getCellContent(fieldRow.getCell(j)), getCellContent(row.getCell(j)));
//            }
//            datas.add(map);
//        }
//        return datas;
//    }
//
//    private static String getCellContent(Cell cell) {
//        if (null == cell) {
//            return null;
//        }
//        StringBuffer buffer = new StringBuffer();
//        switch (cell.getCellType()) {
//            case Cell.CELL_TYPE_NUMERIC: // 数字
//                buffer.append(String.valueOf(cell.getNumericCellValue()));
//                break;
//            case Cell.CELL_TYPE_STRING: // 字符串
//                buffer.append(cell.getStringCellValue());
//                break;
//            case Cell.CELL_TYPE_FORMULA: // 公式
//                buffer.append(cell.getCellFormula());
//                break;
//            default:
//                return null;
//        }
//        return buffer.toString();
//    }

    //    /**
//     * @param map
//     * @param obj
//     * @return
//     * @throws Exception
//     */
//    public static Object parseMap2Bean(Map map, Object obj) throws Exception {
//        try {
//            org.apache.commons.beanutils.BeanUtils.populate(obj, map);
//        } catch (Exception e) {
//            throw new Exception("转换失败！", e);
//        }
//        return obj;
//    }
//
//    /**
//     * @param list
//     * @param
//     * @return
//     * @throws Exception
//     */
//    public static List parseMap2Bean(List<Map> list, Class cla zz) throws Exception {
//        List<Object> lst = new ArrayList<Object>();
//        try {
//            for (Map map : list) {
//                lst.add(parseMap2Bean(map, clazz.newInstance()));
//            }
//        } catch (Exception e) {
//            throw new Exception("转换失败！", e);
//        }
//        return lst;
//    }
    public static void write(HttpServletResponse response, Workbook workbook, String fileName) throws Exception {
        response.setCharacterEncoding("UTF-8");
        response.setContentType("application/vnd.ms-excel;charset=UTF-8");
//        response.setHeader("content-Type", "application/octet-stream");
        if (UtilValidate.isEmpty(fileName)) {
            response.setHeader("Content-Disposition",
                    "attachment;filename=" + URLEncoder.encode("模板文件", "UTF-8") + ".xls");
        } else {
            response.setHeader("Content-Disposition",
                    "attachment;filename=" + URLEncoder.encode(fileName, "UTF-8") + ".xls");
        }
        response.setContentType("*/*");
        ServletOutputStream out = response.getOutputStream();
        workbook.write(out);
        out.flush();
    }

    public static void writeErroData(HttpServletResponse response, List<List<String>> error, String fileName, Class<?> pojo) throws Exception {

        response.setCharacterEncoding("UTF-8");
        response.setContentType("application/vnd.ms-excel;charset=UTF-8");
//        response.setHeader("content-Type", "application/octet-stream");
        if (UtilValidate.isEmpty(fileName)) {
            response.setHeader("Content-Disposition",
                    "attachment;filename=" + System.currentTimeMillis() + ".xls");
        } else {
            response.setHeader("Content-Disposition",
                    "attachment;filename=" + URLEncoder.encode(fileName, "UTF-8") + ".xls");
        }
        response.setContentType("*/*");
        Workbook workbook = new HSSFWorkbook();
        Sheet sheet = workbook.createSheet();
        //设置表头
        Row firstRow = sheet.createRow(0);
        String[] swagger = getSwaggerValue(pojo);
        int i = 0;
        for (i = 0; i < swagger.length; i++) {
            Cell cell = firstRow.createCell(i);
            cell.setCellValue(swagger[i]);
        }
        firstRow.createCell(i).setCellValue("错误原因（修改数据后请删除此列再上传）");
        //

        for (i = 0; i < error.size(); i++) {
            Row row = sheet.createRow(i + 1);
            List<String> rowList = error.get(i);
            for (int j = 0; j < rowList.size(); j++) {
                if (rowList.size() > 10) {
                    if (j == 6 || j == 7) {
                        double month = Double.parseDouble(rowList.get(j));
                        Date date = HSSFDateUtil.getJavaDate(month);

                        row.createCell(j).setCellValue(DateUtils.formatDate(date, "yyyy-MM"));
                    } else {
                        row.createCell(j).setCellValue(rowList.get(j));
                    }
                } else {
                    if (j == 2) {
                        double month = Double.parseDouble(rowList.get(j));
                        Date date = HSSFDateUtil.getJavaDate(month);

                        row.createCell(j).setCellValue(DateUtils.formatDate(date, "yyyy-MM"));
                    } else {
                        row.createCell(j).setCellValue(rowList.get(j));
                    }
                }
            }

        }
        ServletOutputStream out = response.getOutputStream();
        workbook.write(out);
        out.flush();
    }


    public static void downloadTemplate(HttpServletResponse servletResponse, String fileName) throws Exception {
        ClassPathResource classPathResource = new ClassPathResource(fileName + ".xls");
        InputStream inputStream = classPathResource.getInputStream();
        servletResponse.setCharacterEncoding("UTF-8");
        servletResponse.setContentType("application/vnd.ms-excel;charset=UTF-8");
//        response.setHeader("content-Type", "application/octet-stream");
        servletResponse.setHeader("Content-Disposition",
                "attachment;filename=" + URLEncoder.encode("摄像头导入模板", "UTF-8") + ".xls");
        OutputStream out = servletResponse.getOutputStream();
        int length = 0;
        byte[] bytes = new byte[1024];
        while ((length = inputStream.read(bytes)) != -1) {
            out.write(bytes, 0, length);
        }
        out.flush();
        out.close();
        inputStream.close();
    }

    /**
     * 获取excel文件
     *
     * @param multipartFile excel文件
     * @param pojo          转化的实体类
     * @param <T>
     * @return
     * @throws Exception
     */
    public static <T> List<T> readExcel(MultipartFile multipartFile, Class<T> pojo) throws Exception {
        Workbook workbook = getWorkbook(multipartFile);
        List<T> dataList = new ArrayList<>();
        Sheet sheet = workbook.getSheetAt(0);
        //获取实际的行数
        int rowNo = (sheet.getLastRowNum() - sheet.getFirstRowNum()) + 1;
        Field[] fields = pojo.getDeclaredFields();
        //从第三行开始读取数据
        for (int i = 2; i < rowNo; i++) {
            Row row = sheet.getRow(i);
            //校验空行
            if (row == null || row.toString().isEmpty()) {
                break;
            }
            //定义封装数据的实体类
            T t = pojo.newInstance();
            Cell cell = null;
            int cellType = 0;
            for (int j = 0; j < fields.length; j++) {
                try {
                    cell = row.getCell(j);
                    cellType = cell.getCellType();
                } catch (NullPointerException e) {
                    fields[j].set(t, "");
                    continue;
                }
                try {
                    switch (cellType) {
                        case Cell.CELL_TYPE_NUMERIC:
                            if (fields.getClass().getName().equals("String")) {
                                fields[j].set(t, String.valueOf(cell.getNumericCellValue()));
                            }
                            fields[j].set(t, cell.getNumericCellValue());
                            break;
                        case Cell.CELL_TYPE_STRING://字符串类型
                            if (cell.getStringCellValue().equals("null")) {
                                fields[j].set(t, "");
                            } else {
                                fields[j].set(t, cell.getStringCellValue());
                            }
                            break;
                        case CELL_TYPE_BLANK://空单元格类型
                            fields[j].set(t, "");
                            break;
                    }
                } catch (Exception e) {
                    logger.error("数据转换错误");
                }

            }
            dataList.add(t);
        }
        return dataList;
    }
}
