package com.dkha.server.mappers;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.dkha.common.modules.vo.search.SearchPortraitVo;
import com.dkha.server.modules.entities.Portrait;
import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

import java.util.List;
import java.util.Map;

/**
 * <p>
 * 人像表 Mapper 接口
 * </p>
 *
 * @author Spring
 * @since 2019-12-09
 */
@Mapper
public interface PortraitMapper extends BaseMapper<Portrait> {
    /**
     * 分页查询人像表信息
     * @param page
     * @param searchParams
     * @return
     */
    List<Portrait> queryPage(@Param("page") Page page, @Param("searchParams") Map<String, String> searchParams);

    /**
     * 人脸检索分页获取总的记录数量
     * @param searchParams 检索参数
     * @param faceIdList 人脸Id集合--如果上传图片进行搜索，则该值不能为空
     * @param libraryIdList 库Id集合
     * @return
     */
    Integer findSearchPageCount(@Param("searchParams") Map<String, String> searchParams, @Param("faceIdList") List<String> faceIdList, @Param("libraryIdList") List<String> libraryIdList);

    /**
     * 人脸检索获取某一页对应的详细数据
     * @param searchParams 检索参数
     * @param faceIdList 人脸Id集合--如果上传图片进行搜索，则该值不能为空
     * @param libraryIdList 库Id集合
     * @return
     */
    List<SearchPortraitVo> findSearchPageList(@Param("searchParams") Map<String, String> searchParams, @Param("faceIdList") List<String> faceIdList, @Param("libraryIdList") List<String> libraryIdList);

    /**
     * 人脸检索分页--非api调用
     * @param page
     * @param searchParams 检索参数
     * @param faceIdList 人脸Id集合--如果上传图片进行搜索，则该值不能为空
     * @param libraryIdList 库Id集合
     * @return
     */
    List<SearchPortraitVo> findSearchPage(@Param("page") Page<SearchPortraitVo> page, @Param("searchParams") Map<String, String> searchParams, @Param("faceIdList") List<String> faceIdList, @Param("libraryIdList") List<String> libraryIdList);

    /**
     * 根据id查询
     * @param idPortrait
     * @return
             */
    Portrait queryPortrait(@Param("idPortrait") Long idPortrait);

    /**
     * 根据库id查询
     * @param idFactory
     * @return
     */
    List<Portrait> queryPortraitByFactory(@Param("idFactory") Long idFactory);

    /**
     * 根据id删除
     * @param taskID
     * @return
     */
    Integer deletePortrait(@Param("taskID") Long taskID) ;
}
